@@ -71,15 +71,16 @@
 #    endif
 #endif
 
+/* Not strictly needed on non-Solaris, but strcasecmp is POSIX.1-2001 */
+#define _XOPEN_SOURCE   500
 
 /* General includes */
+#include <stdarg.h>
 #include <stdio.h>
 #include <stdlib.h>
-#include <sys/types.h>
+#include <string.h>
 #include <strings.h>
-
 #include <unistd.h>
-#include <errno.h>
 
 #include <gssapi.h>
 #include <globus_gss_assist.h>
@@ -104,8 +105,6 @@ EXTERN_C_BEGIN
 
 /* prototypes */
 globus_result_t lcmaps_callout( va_list ap);
-static int llgt_run_lcas_authz_and_lcmaps_mapping (gss_ctx_id_t context, char *desired_identity, char ** local_identity);
-
 /* -------- */
 
 /***************************************************
@@ -116,11 +115,14 @@ arguments:      gss_ctx_id_t context    - A context with all the credentials in
 Return:         1 : Error
                 0 : All is well
 ****************************************************/
-static int llgt_run_lcas_authz_and_lcmaps_mapping (gss_ctx_id_t context, char *desired_identity, char ** local_identity)
+static int llgt_run_lcas_authz_and_lcmaps_mapping(
+	gss_cred_id_t user_cred_handle,
+	char         *client_name,
+	char         *desired_identity,
+	char        **local_identity,
+	int           sharing)
 {
     int              retval             = 0;
-    gss_cred_id_t    user_cred_handle   = GSS_C_NO_CREDENTIAL;
-    char            *client_name        = NULL;
     FILE            *flog_lcmaps        = NULL;
 #ifndef DISABLE_LCAS
     FILE            *flog_lcas          = NULL;
@@ -128,10 +130,6 @@ static int llgt_run_lcas_authz_and_lcmaps_mapping (gss_ctx_id_t context, char *d
     int              do_run_lcas        = 1; /* Run LCAS by default */
 #endif
 
-    /* get user credential from context */
-    user_cred_handle = llgt_get_user_cred_handle(context);
-    client_name=llgt_get_client_name(context);
-
 #ifndef DISABLE_LCAS
     /* Selectively run LCAS or not. Use "disabled" or "no" as value */
     if ((run_lcas = getenv("LLGT_RUN_LCAS"))) {
@@ -162,7 +160,6 @@ static int llgt_run_lcas_authz_and_lcmaps_mapping (gss_ctx_id_t context, char *d
         /* Execute LCAS with GLOBUS INTERFACE TO LCAS */
         if (llgt_run_lcas (user_cred_handle, client_name, flog_lcas)) {
             llgt_logmsg(LOG_ERR, "Execution of LCAS failed.\n");
-            free(client_name);
             retval = 1;
             goto finalize;
         } else {
@@ -189,16 +186,18 @@ static int llgt_run_lcas_authz_and_lcmaps_mapping (gss_ctx_id_t context, char *d
     llgt_create_jobid();
 
     /* Execute LCMAPS */
-    if (llgt_run_lcmaps(user_cred_handle, client_name, flog_lcmaps, desired_identity, local_identity)) {
+    if (llgt_run_lcmaps(user_cred_handle,
+			client_name,
+			flog_lcmaps,
+			desired_identity,
+			local_identity,
+			sharing))
+    {
         llgt_logmsg(LOG_ERR, "Execution of LCMAPS failed.\n");
-        free(client_name);
         retval = 1;
         goto finalize;
     }
 
-    /* No longer need client_name */
-    free(client_name);
-
     /* Implicit post-LCMAPS mapping to 'root' protection */
     if ((getuid() == 0) || geteuid() == 0 || getgid() == 0 || getegid() == 0) {
         /* Override the Implicit post-LCMAPS mapping to 'root' protection */
@@ -259,6 +258,8 @@ globus_result_t lcmaps_callout( va_list ap)
     char *                              desired_identity        = NULL;
     char *                              identity_buffer         = NULL;
     unsigned int                        buffer_length           = 0;
+    /* Next one is for sharing service only */
+    char *                              shared_user_pem		= NULL;
 
     /* pointer to a buffer with local user name allocated by SAML subroutines */
     /* result of LCMAPS callout, cached to be able to reuse it: globus callout
@@ -266,17 +267,18 @@ globus_result_t lcmaps_callout( va_list ap)
      * is allowed (userok) */
     static char *                       local_identity          = NULL;
     globus_result_t                     result                  = GLOBUS_SUCCESS;
-    gss_name_t                          peer                    = NULL;
-    gss_buffer_desc                     peer_name_buffer; /* This is a struct */
-    gss_name_t                          own                     = NULL;
-    gss_buffer_desc                     own_name_buffer; /* This is a struct */
-    OM_uint32                           major_status            = 0;
-    OM_uint32                           minor_status            = 0;
     int                                 rc                      = 0;
-    int                                 initiator               = 0;
+    gss_cred_id_t			user_cred_handle	= GSS_C_NO_CREDENTIAL;
+    char *				client_name		= NULL;
     
     /* whether to reuse the result from a previous callout */
     char *                              llgt_cache_callout      = NULL;
+    
+    /* Whether we are using the sharing service: then need to use received
+     * pemstring instead of GSS credential */
+    int					sharing			= 0;
+    /* version for the GLOBUS_GSI_GSS_ASSIST_MODULE / GSS Assist */
+    globus_version_t version;
 
     /* gss_priv_attribute_set              attributes;                */
     /* gss_id_set                          attribute_types_required; */
@@ -294,6 +296,36 @@ globus_result_t lcmaps_callout( va_list ap)
     /* Setup logging for this plugin */
     llgt_open_log();
 
+    /* When we are using the sharing service, we get a PEM file as extra
+     * argument */
+    if(strcmp(service, "sharing") == 0)	{
+	/* Test whether we use a version which actually supports sharing,
+	 * otherwise the va_arg will cause a crash */
+	rc=globus_module_get_version(GLOBUS_GSI_GSS_ASSIST_MODULE,&version);
+	if (rc!=GLOBUS_SUCCESS) {
+	    llgt_logmsg(LOG_ERR,
+		    "Cannot get version for GLOBUS_GSI_GSS_ASSIST_MODULE.\n");
+	    result=(globus_result_t)GLOBUS_FAILURE;
+	    goto llgt_finalize;
+	}
+	llgt_logmsg(LOG_DEBUG, "GSS Assist major/minor version=%d/%d\n",
+		version.major, version.minor);
+	/* Minimal version of the globus_gss_assist is 9, see
+	 * http://jira.globus.org/browse/GT-478 */
+	if ( version.major<9 ) {
+	    llgt_logmsg(LOG_ERR,
+		"Requesting service is \"sharing\" but library "
+		"does not support it (major/minor=%d/%d).\n",
+		version.major, version.minor);
+	    result=(globus_result_t)GLOBUS_FAILURE;
+	    goto llgt_finalize;
+	}
+	/* We can continue */
+	sharing=1;
+	shared_user_pem = va_arg(ap, char *);
+    }
+
+
     /* load required modules */
     rc = globus_module_activate(GLOBUS_GRIDMAP_CALLOUT_ERROR_MODULE);
     if(rc!=0) {
@@ -341,12 +373,12 @@ globus_result_t lcmaps_callout( va_list ap)
 	    GLOBUS_GRIDMAP_CALLOUT_ERROR(
 		    result,
 		    GLOBUS_GRIDMAP_CALLOUT_BUFFER_TOO_SMALL,
-		    ("Local identity length: %d, Buffer length: %d\n",
-			strlen(local_identity), buffer_length));
+		    ("Local identity length: %lu, Buffer length: %u\n",
+			(long unsigned)strlen(local_identity), buffer_length));
 	    /* Log error */
 	    llgt_logmsg(LOG_ERR,
-		    "Buffer length %d too small for identity %s",
-		    buffer_length, local_identity);
+		    "Buffer length %lu too small for identity %s",
+		    (long unsigned)buffer_length, local_identity);
 	    /* Reset failed local identity */
 	    free(local_identity);
 	    local_identity=NULL;
@@ -363,82 +395,29 @@ globus_result_t lcmaps_callout( va_list ap)
 	local_identity=NULL;
     }
 
-    /* Need to do actual run: start necessary GSS modules */
-    rc = globus_module_activate(GLOBUS_GSI_GSS_ASSIST_MODULE);
-    if(rc!=0) {
-        llgt_logmsg(LOG_ERR, "Error activating Globus GSS ASSIST MODULE");
-        result = (globus_result_t)GLOBUS_FAILURE;
-        goto llgt_finalize;
+    /* On which input do we need to base our mapping */
+    if (sharing)    {
+	/* Need to build up the gss_cred_id_t from the pemstring */
+	result=llgt_pem_to_gsscred(shared_user_pem, &user_cred_handle, &client_name);
+	if (result!=GLOBUS_SUCCESS)
+	    goto llgt_finalize;
+    } else {
+	/* get GSS user credential from context */
+	user_cred_handle = llgt_get_user_cred_handle(context);
+	/* get client DN from context */
+	result=llgt_get_client_name(context,&client_name);
+	if ( result != GLOBUS_SUCCESS)
+	    goto llgt_finalize;
     }
 
-    rc = globus_module_activate(GLOBUS_GSI_GSSAPI_MODULE);
-    if(rc!=0) {
-        llgt_logmsg(LOG_ERR, "Error activating Globus GSSAPI MODULE");
-        result = (globus_result_t)GLOBUS_FAILURE;
-        goto llgt_finalize;
-    }
-
-    /* find out who is the initiator of this connection (typically the client) */
-    major_status = gss_inquire_context(&minor_status,
-                                       context,
-                                       GLOBUS_NULL,
-                                       GLOBUS_NULL,
-                                       GLOBUS_NULL,
-                                       GLOBUS_NULL,
-                                       GLOBUS_NULL,
-                                       &initiator,
-                                       GLOBUS_NULL);
-    if(GSS_ERROR(major_status)) {
-        GLOBUS_GRIDMAP_CALLOUT_GSS_ERROR(result, major_status, minor_status);
-        llgt_logmsg(LOG_ERR, "Error inquiring context to find the initiator");
-        goto llgt_finalize;
-    }
-
-    /* get our and the peer's name */
-    major_status = gss_inquire_context(&minor_status,
-                                       context,
-                                       initiator ? &own : &peer,
-                                       initiator ? &peer : &own,
-                                       GLOBUS_NULL,
-                                       GLOBUS_NULL,
-                                       GLOBUS_NULL,
-                                       GLOBUS_NULL,
-                                       GLOBUS_NULL);
-    if(GSS_ERROR(major_status)) {
-        GLOBUS_GRIDMAP_CALLOUT_GSS_ERROR(result, major_status, minor_status);
-        llgt_logmsg(LOG_ERR, "Error inquiring context to extract the identity of the peer");
-        goto llgt_finalize;
-    }
-
-    major_status = gss_display_name(&minor_status,
-                                    peer,
-                                    &peer_name_buffer,
-                                    GLOBUS_NULL);
-    if(GSS_ERROR(major_status)) {
-        GLOBUS_GRIDMAP_CALLOUT_GSS_ERROR(result, major_status, minor_status);
-        gss_release_name(&minor_status, &peer);
-        goto llgt_finalize;
-    }
-
-    gss_release_name(&minor_status, &peer);
-
-    major_status = gss_display_name(&minor_status,
-                                    own,
-                                    &own_name_buffer,
-                                    GLOBUS_NULL);
-    if(GSS_ERROR(major_status)) {
-        GLOBUS_GRIDMAP_CALLOUT_GSS_ERROR(result, major_status, minor_status);
-        gss_release_name(&minor_status, &own);
-        goto llgt_finalize;
-    }
-
-    gss_release_name(&minor_status, &own);
-
-
     /*************************
      * Start AuthZ with LCAS *
      *************************/
-    if (llgt_run_lcas_authz_and_lcmaps_mapping (context, desired_identity, &local_identity)) {
+    if (llgt_run_lcas_authz_and_lcmaps_mapping (user_cred_handle,
+						client_name,
+						desired_identity,
+						&local_identity,
+						sharing)) {
         /* llgt_logmsg(LOG_ERR, "Failure in Authorization and Mapping"); */
         result = (globus_result_t)GLOBUS_FAILURE;
         goto llgt_finalize;
@@ -452,13 +431,13 @@ globus_result_t lcmaps_callout( va_list ap)
         GLOBUS_GRIDMAP_CALLOUT_ERROR(
 		result,
 		GLOBUS_GRIDMAP_CALLOUT_BUFFER_TOO_SMALL,
-                ("Local identity length: %d Buffer length: %d\n",
-		 strlen(local_identity),
+                ("Local identity length: %lu Buffer length: %u\n",
+		 (long unsigned)strlen(local_identity),
 		 buffer_length)
 		);
         llgt_logmsg(LOG_ERR,
-		"Failed map user. Buffer to small: local identity length: %d Buffer length: %d",
-		strlen(local_identity),
+		"Failed map user. Buffer to small: local identity length: %lu Buffer length: %u",
+		(long unsigned)strlen(local_identity),
 		buffer_length);
     } else {
         strcpy(identity_buffer, local_identity);
@@ -467,9 +446,6 @@ globus_result_t lcmaps_callout( va_list ap)
 		service ? service : "unknown", identity_buffer);
     }
 
-    gss_release_buffer(&minor_status, &peer_name_buffer);
-    gss_release_buffer(&minor_status, &own_name_buffer);
-
     result = GLOBUS_SUCCESS;
 
     /* syslog log closing */
@@ -479,6 +455,10 @@ llgt_finalize:
     llgt_close_log();
 
     /* free strings that have possibly been allocated */
+    if (sharing)
+	llgt_cleanup_gss_cred_id_t(user_cred_handle);
+    free(client_name);
+
     /* if they werent allocated they will be NULL and this does no harm */
 /*    free(local_identity);*/ /* Do not free, we will reuse it in the userok
 				 call */
