@@ -37,12 +37,19 @@
  *
  */
 
+#define _XOPEN_SOURCE	600
+
+#include <dlfcn.h>
 #include <stdio.h>
 #include <stdlib.h>
-#include <dlfcn.h>
+#include <string.h>
+#include <strings.h>
 #include <syslog.h>
+#include <unistd.h>
 #include <sys/stat.h>
-#include <lcas/lcas.h>
+#include <sys/types.h>
+#include <errno.h>
+/* #include <lcas/lcas.h> */
 
 #include "llgt_config.h"
 
@@ -77,25 +84,35 @@ static int set_liblcas_path(char ** liblcas_path)
             llgt_logmsg(LOG_WARNING, "%s: Ignoring $LLGT_LCAS_LIBDIR as \"%s\" is not an absolute path to a valid directory\n", __func__, lcas_libdir);
             lcas_libdir = LCAS_LIBDIR;
         }
-        /* Get the potentially different $LCAS_MODULE_SFX, based on the customized libdir */
+	/* Get the potentially different $LCAS_MODULE_SFX, based on the
+	 * customized libdir */
         lcas_moddir_sfx = getenv("LLGT_LCAS_MODULEDIR_SFX");
-        if (!lcas_moddir_sfx) {
-            lcas_moddir_sfx = LCAS_MODULEDIR_SFX; /* Take the input from the LLGT configure */
-        }
+        if (!lcas_moddir_sfx)
+	    /* Take the input from the LLGT configure */
+            lcas_moddir_sfx = LCAS_MODULEDIR_SFX;
 
-        /* Concatting LCAS_LIBDIR and LCAS_MODULEDIR_SFX to be exported as $LCAS_MODULES_DIR */
+	/* Concatting LCAS_LIBDIR and LCAS_MODULEDIR_SFX to be exported as
+	 * $LCAS_MODULES_DIR */
         lcas_modules_dir = malloc(AP_MAXPATH);
         if (!lcas_modules_dir) {
-            llgt_logmsg(LOG_ERR, "%s: Could not allocate memory: %s\n", __func__, strerror(errno));
+            llgt_logmsg(LOG_ERR, "%s: Could not allocate memory: %s\n",
+		    __func__, strerror(errno));
             return 1;
         } else {
-	    sn_retval=snprintf(lcas_modules_dir,AP_MAXPATH,"%s%s", lcas_libdir, lcas_moddir_sfx);
+	    sn_retval=snprintf(lcas_modules_dir,AP_MAXPATH,"%s%s",
+		    lcas_libdir, lcas_moddir_sfx);
 	    if (sn_retval<0)	{
-		llgt_logmsg(LOG_WARNING, "snprintf failed when creating full modulespath, not setting LCAS_MODULES_DIR\n");
+		llgt_logmsg(LOG_WARNING,
+			"snprintf failed when creating full modulespath, "
+			"not setting LCAS_MODULES_DIR\n");
 	    } else if ((size_t)sn_retval >= AP_MAXPATH) {
-                llgt_logmsg(LOG_WARNING, "Full modulespath '%s%s' would be too long, not setting LCAS_MODULES_DIR\n", lcas_libdir,lcas_moddir_sfx);
+                llgt_logmsg(LOG_WARNING,
+			"Full modulespath '%s%s' would be too long, "
+			"not setting LCAS_MODULES_DIR\n",
+			lcas_libdir,lcas_moddir_sfx);
             } else {
-                llgt_logmsg(LOG_DEBUG, "Setting LCAS_MODULES_DIR to '%s'\n", lcas_modules_dir);
+                llgt_logmsg(LOG_DEBUG, "Setting LCAS_MODULES_DIR to '%s'\n",
+			lcas_modules_dir);
                 setenv("LCAS_MODULES_DIR", lcas_modules_dir, 1);
             }
             free(lcas_modules_dir);
@@ -103,16 +120,21 @@ static int set_liblcas_path(char ** liblcas_path)
 
         /* Setting the full path to the liblcas.so object */
         *liblcas_path = malloc(AP_MAXPATH);
-	sn_retval=snprintf(*liblcas_path, AP_MAXPATH, "%s/%s", lcas_libdir, LIBLCAS_SO);
+	sn_retval=snprintf(*liblcas_path, AP_MAXPATH, "%s/%s",
+		lcas_libdir, LIBLCAS_SO);
 	if (sn_retval<0)    {
-            llgt_logmsg(LOG_ERR, "snprintf failed when creating full path to lcas lib %s\n",LIBLCAS_SO);
+            llgt_logmsg(LOG_ERR,
+		    "snprintf failed when creating full path to lcas lib %s\n",
+		    LIBLCAS_SO);
             return 1;
 	} else if ((size_t)sn_retval >= AP_MAXPATH) {
-            llgt_logmsg(LOG_ERR, "Full path to %s \"%s/%s\" is too long\n", LIBLCAS_SO, lcas_libdir, LIBLCAS_SO);
+            llgt_logmsg(LOG_ERR, "Full path to %s \"%s/%s\" is too long\n",
+		    LIBLCAS_SO, lcas_libdir, LIBLCAS_SO);
             return 1;
         }
     } else {
-        /* Just set the name of the shared object file, let the system figure it out */
+	/* Just set the name of the shared object file, let the system figure it
+	 * out */
         *liblcas_path = strdup(LCAS_LIBDIR LIBLCAS_SO);
     }
     llgt_logmsg(LOG_DEBUG, "LCAS library path : \"%s\"\n", *liblcas_path);
@@ -128,7 +150,8 @@ void llgt_setup_lcas_environment(void)
     if (!getenv ("LCAS_DB_FILE"))
         setenv("LCAS_DB_FILE", SYSCONFDIR"/lcas/lcas.db", 1);
 
-/* Not required by default, because the setting of the LCMAPS_LOG_FILE will declare to log to file, or use Syslog instead */
+/* Not required by default, because the setting of the LCAS_LOG_FILE will
+ * declare to log to file, or use Syslog instead */
 #ifdef LCAS_LCMAPS_FORCE_LOG_TO_FILE
     if (!getenv ("LCAS_LOG_FILE"))
         setenv ("LCAS_LOG_FILE",  "/var/log/gt_lcas_lcmaps.log", 1);
@@ -156,26 +179,29 @@ int llgt_run_lcas(gss_cred_id_t user_cred_handle, char * client_name, FILE * log
 
     int rc=1; /* Return code is error unless success */
     char * error        = NULL;
-    int    retval       = 0;
 
     /* If there is no PEM String, nothing to do */
     if (!user_cred_handle)
     {
-	llgt_logmsg(LOG_ERR, "Couldn't extract the client certificate credentials.\n");
+	llgt_logmsg(LOG_ERR,
+		"Couldn't extract the client certificate credentials.\n");
 	return 1;
     }
 
     /* Check we don't have a handle yet */
     if (LcasHandle==NULL)	{
-	/* Get the path to the to be opened LCAS main library - Must be free'd */
+	/* Get the path to the to be opened LCAS main library - Must be free'd
+	 * */
 	if (set_liblcas_path(&(liblcas_path))) {
-	    llgt_logmsg(LOG_ERR, "Couldn't set the path to \"%s\"\n", LIBLCAS_SO);
+	    llgt_logmsg(LOG_ERR, "Couldn't set the path to \"%s\"\n",
+		    LIBLCAS_SO);
 	    return 1;
 	}
 
 	/* I must have a path set */
 	if (!(liblcas_path)) {
-	    llgt_logmsg(LOG_ERR, "Failed set a name or path to find liblcas.so\n");
+	    llgt_logmsg(LOG_ERR,
+		    "Failed set a name or path to find liblcas.so\n");
 	    return 1;
 	}
 
@@ -184,7 +210,7 @@ int llgt_run_lcas(gss_cred_id_t user_cred_handle, char * client_name, FILE * log
 
 	if (!LcasHandle) {
 	    llgt_logmsg(LOG_ERR,
-		"Failed to dynamically load the library for LCAS: \"%s\"\n",
+		"Failed to dynamically load the library for LCAS %s: \"%s\"\n",
 		liblcas_path,dlerror());
 	    free(liblcas_path); liblcas_path=NULL;
 	    return 1;
@@ -197,21 +223,25 @@ int llgt_run_lcas(gss_cred_id_t user_cred_handle, char * client_name, FILE * log
     LcasInit=dlsym(LcasHandle,"lcas_init");
     if ((error = dlerror()) != NULL)
     {
-        llgt_logmsg(LOG_ERR, "LCAS module not compliant: Symbol \"lcas_init\" not found: %s\n", error);
+        llgt_logmsg(LOG_ERR,
+	    "LCAS module not compliant: "
+	    "Symbol \"lcas_init\" not found: %s\n", error);
 	llgt_lcas_terminate(0);
 	return 1;
     }
     LcasGetFabricAuthorization=dlsym(LcasHandle,"lcas_get_fabric_authorization");
     if ((error = dlerror()) != NULL)
     {
-        llgt_logmsg(LOG_ERR, "LCAS module not compliant: Symbol \"lcas_get_fabric_authorization\" not found: %s\n", error);
+        llgt_logmsg(LOG_ERR,
+	    "LCAS module not compliant: "
+	    "Symbol \"lcas_get_fabric_authorization\" not found: %s\n", error);
 	llgt_lcas_terminate(0);
 	return 1;
     }
 
     /* Initialize, send authorization request to and terminate the LCAS */
     /* retval=lcas_init(config->usrlog_fp); */
-    if ((retval = (*LcasInit)(logfile)))
+    if ( (*LcasInit)(logfile) != 0)
     {
         llgt_logmsg(LOG_ERR, "LCAS initialization failure.\n");
 	/* no need for term, can safely close off */
@@ -220,15 +250,17 @@ int llgt_run_lcas(gss_cred_id_t user_cred_handle, char * client_name, FILE * log
     }
 
     /* Authorization based on the GSI input */
-    if ((retval=(*LcasGetFabricAuthorization)(client_name, user_cred_handle, "RSL") != 0))
+    if ( (*LcasGetFabricAuthorization)(client_name,user_cred_handle,"RSL")!=0)
     {
-        llgt_logmsg(LOG_WARNING, "%s: The user is not authorized by LCAS.\n", __func__);
+        llgt_logmsg(LOG_WARNING, "%s: The user is not authorized by LCAS.\n",
+		__func__);
 	/* LCAS failed */
 	rc=1;
     }
     else
     {
-        llgt_logmsg(LOG_INFO, "%s: The user is authorized by LCAS.\n", __func__);
+        llgt_logmsg(LOG_INFO, "%s: The user is authorized by LCAS.\n",
+		__func__);
 	/* LCAS succeeded */
 	rc=0;
     }
@@ -261,7 +293,9 @@ int llgt_lcas_terminate(int call_term)	{
 	LcasTerm=dlsym(LcasHandle,"lcas_term");
 	if ((error = dlerror()) != NULL)
 	{
-	    llgt_logmsg(LOG_ERR, "LCAS module not compliant: Symbol \"lcas_term\" not found: %s\n", error);
+	    llgt_logmsg(LOG_ERR,
+		    "LCAS module not compliant: "
+		    "Symbol \"lcas_term\" not found: %s\n", error);
 	    retval=1;
 	} else {
 	    /* Terminate LCAS framework */
