@@ -39,7 +39,7 @@
 
 
 #ifndef LLGT_LCMAPS_H
-#define LLGT_LCMAPS_H
+# define LLGT_LCMAPS_H
 
 #include <stdio.h>
 #include <gssapi.h>
@@ -53,27 +53,32 @@
 /* Default db file is set in configure */
 
 #ifndef LCMAPS_LOG_FILE
-#define LCMAPS_LOG_FILE "/var/log/glexec/lcas_lcmaps.log"
+# define LCMAPS_LOG_FILE "/var/log/glexec/lcas_lcmaps.log"
 #endif
 
 #ifndef LCMAPS_DEBUG_LEVEL
-#define LCMAPS_DEBUG_LEVEL   "4"
+# define LCMAPS_DEBUG_LEVEL   "4"
 #endif
 
 /* LCMAPS_LIBDIR is used to search the liblcmaps*.so files, when set to "", the
  * system default search path is used (ld.so.conf etc.) */
 #ifndef LCMAPS_LIBDIR
-#define LCMAPS_LIBDIR	""
+# define LCMAPS_LIBDIR	""
 #endif
 
 #ifndef LCMAPS_POLICY_NAME
-#define LCMAPS_POLICY_NAME  ""
+# define LCMAPS_POLICY_NAME  ""
 #endif
 
 /* Prototypes */
 
 void llgt_setup_lcmaps_environment(void);
-int llgt_run_lcmaps (gss_cred_id_t, char *, FILE *, char *, char **);
+int llgt_run_lcmaps(gss_cred_id_t user_cred_handle,
+		    char         *client_name,
+		    FILE         *logfile,
+		    char         *desired_username,
+		    char        **username,
+		    int           sharing);
 
 #endif /* LLGT_LCMAPS_H */
 
